package com.dhalucario.HeartContainersReloaded;

import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.configuration.MemorySection;
import org.bukkit.entity.*;

public class HCConfigLoader {

    JavaPlugin hcPlugin;

    public HCConfigLoader(JavaPlugin plugin) {
        hcPlugin = plugin;
    }

    public void loadDefaults() {

        hcPlugin.getConfig().options().copyDefaults(true);

        hcPlugin.getConfig().addDefault("start_health", 3.0);
        hcPlugin.getConfig().addDefault("max_health", 30.0);
        hcPlugin.getConfig().addDefault("spawning_resets_health", true);

        for (EntityType entType : EntityType.values()) {

            /*
             * drop_value: Drops a heart which contains the value specified using health_value
             * drop_vampire: Drops a heart which contains the (health of the target * vampire_multiplicator)
             * direct_value: Adds the value specified in health_value directly to the max health
             * direct_vampire:  Adds the value of (health of the target * vampire_multiplicator) directly to the max health
             */
            hcPlugin.getConfig().addDefault("health_config." + entType.name() + ".health_type", "NONE");
            hcPlugin.getConfig().addDefault("health_config." + entType.name() + ".health_value", 0.5);
            hcPlugin.getConfig().addDefault("health_config." + entType.name() + ".vampire_multiplicator", 0.5);
        }

        hcPlugin.saveDefaultConfig();
    }

    public HCEntityConfig getConfigForEntity(EntityType target) {

        HCEntityConfig entityConfig = new HCEntityConfig();
        MemorySection dropConfig = (MemorySection) hcPlugin.getConfig().get("health_config");
        MemorySection damageEntityConfig = (MemorySection) dropConfig.get(target.name());

        if (damageEntityConfig != null) {
            if (damageEntityConfig.isString("health_type")) {

                switch (damageEntityConfig.getString("health_type")) {
                    case "DROP_VALUE":
                        entityConfig.setHealthType(HealthType.DROP_VALUE);
                        break;
                    case "DROP_VAMPIRE":
                        entityConfig.setHealthType(HealthType.DROP_VAMPIRE);
                        break;
                    case "DIRECT_VALUE":
                        entityConfig.setHealthType(HealthType.DIRECT_VALUE);
                        break;
                    case "DIRECT_VAMPIRE":
                        entityConfig.setHealthType(HealthType.DIRECT_VAMPIRE);
                        break;
                    case "NONE":
                        entityConfig.setHealthType(HealthType.NONE);
                        break;
                    default:
                        hcPlugin.getLogger().warning("\"health_type\" for entity " + target.name() + " is an invalid type");
                        hcPlugin.getLogger().warning("Falling back to default (0.5)");
                        entityConfig.setHealthType(HealthType.DROP_VALUE);
                        break;
                }
            } else {
                entityConfig.setVampireMultiplicator(0.5);
                hcPlugin.getLogger().warning("\"health_type\" for entity " + target.name() + " is an invalid primitive");
                hcPlugin.getLogger().warning("Falling back to default (0.5)");
            }

            if (damageEntityConfig.isDouble("health_value")) {
                entityConfig.setHealthValue(damageEntityConfig.getDouble("health_value"));
            } else if (damageEntityConfig.isInt("health_value")) {
                entityConfig.setHealthValue(damageEntityConfig.getInt("health_value"));
            } else {
                entityConfig.setHealthValue(damageEntityConfig.getInt("health_value"));
                hcPlugin.getLogger().warning("\"health_value\" for entity " + target.name() + " is an invalid primitive");
                hcPlugin.getLogger().warning("Falling back to default (0.5)");
            }

            if (damageEntityConfig.isDouble("vampire_multiplicator")) {
                entityConfig.setVampireMultiplicator(damageEntityConfig.getDouble("vampire_multiplicator"));
            } else if (damageEntityConfig.isInt("vampire_multiplicator")) {
                entityConfig.setVampireMultiplicator((double) damageEntityConfig.getInt("vampire_multiplicator"));
            } else {
                entityConfig.setVampireMultiplicator(0.5);
                hcPlugin.getLogger().warning("\"vampire_multiplicator\" for entity " + target.name() + " is an invalid primitive");
                hcPlugin.getLogger().warning("Falling back to default (0.5)");
            }
        } else {
            entityConfig = null;
        }

        return entityConfig;

    }

    public boolean playerRespawnResetsHealth() {
        boolean respawnResetsHealth = false;

        if (hcPlugin.getConfig().isBoolean("spawning_resets_health")) {
            respawnResetsHealth = hcPlugin.getConfig().getBoolean("spawning_resets_health");
        } else {
            respawnResetsHealth = true;
            hcPlugin.getLogger().warning("\"spawning_resets_health\" is an invalid primitive");
            hcPlugin.getLogger().warning("Falling back to default (true)");
        }

        return respawnResetsHealth;
    }

    public double getPlayerRespawnHealth() {

        double respawnHealth = 0;

        if (hcPlugin.getConfig().isDouble("start_health")) {
            respawnHealth = hcPlugin.getConfig().getDouble("start_health");
        } else if (hcPlugin.getConfig().isInt("start_health")) {
            respawnHealth = (double) hcPlugin.getConfig().getInt("start_health");
        } else {
            respawnHealth = 30;
            hcPlugin.getLogger().warning("\"start_health\" is an invalid primitive");
            hcPlugin.getLogger().warning("Falling back to default (true)");
        }

        return respawnHealth;

    }

    public void save() {
        hcPlugin.saveDefaultConfig();
    }

}
