package com.dhalucario.HeartContainersReloaded;

import java.util.ArrayList;

import org.bukkit.Material;
import org.bukkit.attribute.Attribute;
import org.bukkit.entity.Entity;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityPickupItemEvent;
import org.bukkit.event.entity.ItemDespawnEvent;
import org.bukkit.event.player.PlayerRespawnEvent;
import org.bukkit.inventory.ItemStack;

public class HCController implements Listener {

    JavaPlugin hcPlugin;
    HCConfigLoader hcConfigLoader;
    ArrayList<HCItem> hcItemList;

    public HCController(JavaPlugin plugin, HCConfigLoader configLoader) {
        hcPlugin = plugin;
        hcConfigLoader = configLoader;
        hcItemList = new ArrayList<HCItem>();
    }

    @EventHandler
    public void onPlayerRespawn(PlayerRespawnEvent e) {
        if (hcConfigLoader.playerRespawnResetsHealth()) {
            e.getPlayer().getAttribute(Attribute.GENERIC_MAX_HEALTH).setBaseValue(hcConfigLoader.getPlayerRespawnHealth());
        }
    }

    @EventHandler
    public void onDamage(EntityDamageByEntityEvent e) {

        Entity damager = e.getDamager();
        Entity damagee = e.getEntity();
        double damage = e.getDamage();

        if (damager instanceof Player && damagee instanceof LivingEntity) {

            LivingEntity target = (LivingEntity) damagee;
            if ((target.getHealth() - damage) <= 0) {
                handleHeartContainer("death", (Player) damager, target);
            }
        }
    }

    @EventHandler
    public void onPlayerPickup(EntityPickupItemEvent e) {
        //Todo: Check if hearts are picked up

        if (e.getEntity() instanceof Player) {

            Player pickupPlayer = (Player) e.getEntity();
            int eventItemId = e.getItem().getEntityId();

            for (HCItem hcItem : hcItemList) {
                if (eventItemId == hcItem.healthItem.getEntityId()) {
                    handleHeartContainer("pickup", pickupPlayer, hcItem.healthTarget);
                    e.getItem().remove();
                    e.setCancelled(true);
                }
            }

        }
    }

    public void onItemDespawn(ItemDespawnEvent e) {

        ArrayList<HCItem> tempRemoveList = new ArrayList<HCItem>();

        for (HCItem hcItem : hcItemList) {
            if (hcItem.healthItem.getEntityId() == e.getEntity().getEntityId()) {
                tempRemoveList.add(hcItem);
            }
        }

        for (HCItem tempItem : tempRemoveList) {
            hcItemList.remove(tempItem);
        }

    }

    void handleHeartContainer(String eventType, Player attacker, LivingEntity target) {

        HCEntityConfig entHealthConfig = hcConfigLoader.getConfigForEntity(target.getType());

        if (eventType == "death") {

            switch (entHealthConfig.getHealthType()) {
                case DROP_VALUE:
                    handleDamageDrop(attacker, target);
                case DROP_VAMPIRE:
                    handleDamageDrop(attacker, target);
                    break;
                case DIRECT_VALUE:
                    handleHealthDirectValue(attacker, entHealthConfig);
                    break;
                case DIRECT_VAMPIRE:
                    handleHealthDirectVampire(attacker, target, entHealthConfig);
                    break;
                default:
                    break;
            }

        } else if (eventType == "pickup") {
            switch (entHealthConfig.getHealthType()) {
                case DROP_VALUE:
                    handleDamageDropPickupValue(attacker, entHealthConfig);
                    break;
                case DROP_VAMPIRE:
                    handleDamageDropPickupVampire(attacker, target, entHealthConfig);
                    break;
                default:
                    break;
            }
        }

    }


    void handleDamageDrop(Player attacker, LivingEntity target) {
        ItemStack healthDrop = new ItemStack(Material.RED_DYE);

        HCItem hcItem = new HCItem();
        hcItem.setHealthTarget(target);
        hcItem.setHealthItemStack(target.getWorld().dropItemNaturally(target.getLocation(), healthDrop));

        hcItemList.add(hcItem);
    }

    void handleDamageDropPickupValue(Player attacker, HCEntityConfig healthConfig) {
        handleHealthDirectValue(attacker, healthConfig);
    }

    void handleDamageDropPickupVampire(Player attacker, LivingEntity target, HCEntityConfig healthConfig) {
        handleHealthDirectVampire(attacker, target, healthConfig);
    }

    void handleHealthDirectValue(Player attacker, HCEntityConfig healthConfig) {
        double attackerHealth = attacker.getAttribute(Attribute.GENERIC_MAX_HEALTH).getValue();
        double additionalHealth = healthConfig.getHealthValue();
        double newHealthValue = attackerHealth + additionalHealth;

        attacker.getAttribute(Attribute.GENERIC_MAX_HEALTH)
                .setBaseValue(attackerHealth + healthConfig.getHealthValue());

        attacker.sendMessage("Your new health value is " + Double.toString(newHealthValue) + " (+" + Double.toString(additionalHealth) + ")");
    }

    void handleHealthDirectVampire(Player attacker, LivingEntity target, HCEntityConfig entConfig) {
        // Todo: Add health directly to the player
        HCEntityConfig healthConfig = hcConfigLoader.getConfigForEntity(target.getType());

        double attackerHealth = attacker.getAttribute(Attribute.GENERIC_MAX_HEALTH).getValue();
        double vampireHealth = getTargetVampireHealth(target, healthConfig);
        double newHealthValue = attackerHealth + vampireHealth;

        attacker.getAttribute(Attribute.GENERIC_MAX_HEALTH)
                .setBaseValue(newHealthValue);

        attacker.sendMessage("Your new health value is " + Double.toString(newHealthValue) + " (+" + Double.toString(vampireHealth) + ")");
    }

    double getTargetVampireHealth(LivingEntity target, HCEntityConfig healthConfig) {
        double targetHealth = target.getAttribute(Attribute.GENERIC_MAX_HEALTH).getValue();
        return (targetHealth * healthConfig.getVampireMultiplicator());
    }

}